

; *** DEATH DERBY ***
;FILENAME	DATE				WORKED ON BY
;DD_2LK_21	01/10/2004			(Glenn Saunders)
;--color gradient in the timer/gear indicator
;--optimized RAM, worked around controller code
;--added sound for scream and engine


;DD_2LK_18	01/06/2004			(Glenn Saunders)
;--Made colors more stable in score area
;--optimized a little bit of the pointer calculation area

;FILENAME	DATE				WORKED ON BY
;DD_2LK_16	03/24/2003			(Glenn Saunders)
;--Adding simple collisions between cars and gremlins
;--Added simple 8-way movement for cars
;--game reset has a bug that needs fixing


;DD_2LK_15	03/10/2003			(Glenn Saunders)
;--game reset code is just about done now.  No color cycling yet.  
;Now it's time to work on incrementing the player scores when gremlins wrap
;work on car movement routines


;DD_2LK_14	03/5/2003			(Glenn Saunders)
;--working on game reset debounce, not completely finished yet


;DD_2LK_13	02/24/2003			(Glenn Saunders)
;--Working timer countdown
;--fixed car glitch bug


;DD_2LK_12	02/24/2003			(Glenn Saunders/Thomas Jentzsch)
;--Thomas fixed the M1 glitch when crossing top boundary.
;--I re-integrated the fix he made to the new codebase.
;--I was then able to finish off the HMMx precompensation routine for M1.
;--Re-integrated stagger-step demo animation for gremlins.



;DD_2LK_10	02/21/2003			(Glenn Saunders)
;--score kernel functional
;--reverse gear indicators finished
;--controller reading code complete.  Cars rotate again finally!!
;--Wrote first 'real' code ever for the Y-cable (for the reverse gear indicators).
;--reading triggers.  Got the cars moving (just up for now).


;DD_2LK_07	02/20/2003			(Glenn Saunders)
;--started working on added reverse gear indicators
;--working on a fully functional score display
;--to do indirect y load of digits



;DD_2LK_06	02/17/2003			(Glenn Saunders)
; -fixed horizontal position of Gremlin0 when cut off at top of screen
; -Gremlin1 still a little messed up
; -got a static score up there, started work on score digit graphics
; -set up top and	bottom borders, colors come from RAM 
; -implemented color/BW switch.  All colors switch to Greyscale
; -implemented typing scheme for border and	grenade for game-state purposes



;DD_2LK_02	 02/10/2003			(Glenn Saunders)
; -standardized labels
; -Removed random tombstone generator
; -created new title screen (not easy!)

; -added initial walkcycle animation for the zombies
; -added hardcoded zigzag movement of zombies (as a demo)

;			01/26/2003:			(Thomas Jentzsch)
; - optimized kernel, *no* tearing of Gremlins and	Cars
; - kernel initialisation code added, all four objects can be freely moved
;	*everywhere* (DEBUG: use left, right joystick and	left fire button)
;	TODO: correct X-position if Gremlin enters at top of screen
; - reduced Gremlin RAM usage
; - halved tombstone RAM usge

;			01/19/2003 (later):	 (Thomas Jentzsch) 
; - improved grenade code:
;	- no safe zone colouring
;	- taler ball
;	- fullscreen ball mode switchable with right difficulty

;			01/19/2003:			(Thomas Jentzsch)
; - first working 2LK demo version with preliminary grenade


;NAMING CONVENTION:
; CONSTANTS (all caps)
; variables (Mixed case)
; subroutines (Mixed case)
; please be verbose, avoid abbreviations and	acronyms
; all should use underscores between logical words (i.e. Car0_X)



	processor 6502
	include vcs.h
	include macro.h

;===============================================================================
; T I A - C O N S T A N T S
;===============================================================================

BM_SIZE_1	= %000000
BM_SIZE_2	= %010000
BM_SIZE_4	= %100000
BM_SIZE_8	= %110000

HMOVE_L7	= $70
HMOVE_L6	= $60
HMOVE_L5	= $50
HMOVE_L4	= $40
HMOVE_L3	= $30
HMOVE_L2	= $20
HMOVE_L1	= $10
HMOVE_0		= $00
HMOVE_R1	= $f0
HMOVE_R2	= $e0
HMOVE_R3	= $d0
HMOVE_R4	= $c0
HMOVE_R5	= $b0
HMOVE_R6	= $a0
HMOVE_R7	= $90
HMOVE_R8	= $80

;JS1_LEFT	 = %00000000
;JS1_RIGHT	 = %00000000
;JS1_UP			= %00000000
;JS1_DOWN	 = %00000000

JS1_LEFT	= %00001000
JS1_RIGHT	= %00000100
JS1_DOWN	= %00000010
JS1_UP		= %00000001
JS1_UP_LEFT	= JS1_UP | JS1_LEFT
JS1_UP_RIGHT	= JS1_UP | JS1_RIGHT
JS1_DOWN_LEFT	= JS1_DOWN | JS1_LEFT
JS1_DOWN_RIGHT	= JS1_DOWN | JS1_RIGHT


DISABLE_BM  	= %00
ENABLE_BM	= %10

;===============================================================================
; U S E R - C O N S T A N T S
;===============================================================================

SPLASH = 1

TICK_LENGTH = 10

;SOUND STUFF
NOISE_ID_SCREAM		= 3
NOISE_ID_CRASH		= 2
NOISE_ID_ENGINE 	= 1
NOISE_ID_SILENCE 	= 0

NOISE_DURATION_SCREAM 	= 60
NOISE_DURATION_CRASH 	= 15




BCD_0 = 0
BCD_1 = 1
BCD_2 = 2
BCD_3 = 3
BCD_4 = 4
BCD_5 = 5
BCD_6 = 6
BCD_7 = 7
BCD_8 = 8
BCD_9 = 9

BCD0_ = 0 << 4
BCD1_ = 1 << 4
BCD2_ = 2 << 4
BCD3_ = 3 << 4
BCD4_ = 4 << 4
BCD5_ = 5 << 4
BCD6_ = 6 << 4
BCD7_ = 7 << 4
BCD8_ = 8 << 4
BCD9_ = 9 << 4

GAME_TIMER_INITIAL 		= BCD0_|BCD_0
GAME_TIMER_START 		= BCD9_|BCD_9

; constants that reference rotation index pointers
PointUp 			= 0
PointRight 			= 4
PointDown 			= 8
PointLeft 			= 12

TurnOnSpriteReflect 		= %00001000
TurnOffSpriteReflect 		= 0

CarPage				= $FF;00
CarDataAlign			= $65
FlipThreshold			= #9


GAME_STATE_START 		= 1
GAME_STATE_OVER 		= 0

GAME_STATE_INITIAL 		= GAME_STATE_OVER; game over

SCREEN_WIDTH			= 160

TOMBSTONE_HEIGHT 		= 8  ; 8*2 = 16 (single scanlines) high per tombstone
TOMBSTONE_ROWS			= 11 ; 11 rows of tombstones per screen

CAR_HEIGHT			= 8  ; 8 doubled scanlines (used as offset)
GREMLIN_HEIGHT			= 8

SCORE_HEIGHT 			= 5

KERNEL_HEIGHT			= TOMBSTONE_ROWS*TOMBSTONE_HEIGHT+CAR_HEIGHT

CAR_Y_INITIAL 			= KERNEL_HEIGHT/2+40
CAR0_X_INITIAL 			= 17
CAR1_X_INITIAL 			= 135

GREMLIN0_X_INITIAL		= 0
GREMLIN1_X_INITIAL		= 160
GREMLIN0_Y_INITIAL		= 210
GREMLIN1_Y_INITIAL		= 140


;ANIMATION
GREMLIN_SPEED			= 2
GREMLIN_FRAMES 			= 5



;INITIALIZATION CONSTANTS...


;INITIAL SETTINGS (COLOR)
CAR0_COLOR_C			= $36;Also colors Gremlin 0
CAR1_COLOR_C			= $c6;Also colors Gremlin 1
FORCEFIELD_GENERATOR_COLOR_C 	= $93
TIMER_COLOR_C	 		= $10


;INITIAL SETTINGS (B&W)
CAR0_COLOR_BW			= $0F;Also colors Gremlin 0
CAR1_COLOR_BW			= $06;Also colors Gremlin 1
FORCEFIELD_GENERATOR_COLOR_BW 	= $02
TIMER_COLOR_BW	 		= $00



;GAME OVER SETTINGS (COLOR)
CAR0_COLOR_C_GO			= $36;Also colors Gremlin 0
CAR1_COLOR_C_GO			= $c6;Also colors Gremlin 1
FORCEFIELD_GENERATOR_COLOR_C_GO = $93
TIMER_COLOR_C_GO	 	= $18


BACKGROUND_COLOR_GAME_OVER	= $04
BACKGROUND_COLOR		= 0


;GAME OVER SETTINGS (B&W)
CAR0_COLOR_BW_GO		= $0F;Also colors Gremlin 0
CAR1_COLOR_BW_GO		= $06;Also colors Gremlin 1
FORCEFIELD_GENERATOR_COLOR_BW_GO = $02
TIMER_COLOR_BW_GO	 	= $0F



;STATE
BORDER_STATE_INITIAL 		= 0;solid

BALL_COLUMN_INITIAL		= $58;This should be moved to RAM
BALL_WIDTH			= BM_SIZE_1/4


RAND_EOR_8	 		= $b2


FACING_LEFT 			= 0
FACING_RIGHT 			= GREMLIN_FRAMES; use as an +X offset to frames of graphics



;SCORE_PAGE = >Score_0

CAR0_GEAR_INDICATOR_REVERSE 	= %00000000
CAR1_GEAR_INDICATOR_REVERSE 	= %00000000

CAR0_GEAR_INDICATOR_FORWARD 	= %01111111
CAR1_GEAR_INDICATOR_FORWARD 	= %11101111

Y_BOUNDARY_BOTTOM		= KERNEL_HEIGHT+80+(CAR_HEIGHT*2)

	

;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

	SEG.U	variables
	ORG	$80

Frame_Counter			ds 1

Random				ds 1

;TEMP VARIABLE OVERLAYS

Overlay0			ds 1
Overlay1			ds 1
Overlay2			ds 1
Car_Picker			ds 1
OverlayDouble			ds 2

Temp_Var			= Overlay0
Gremlin_Pointer_Temp 		= Overlay0
P0_P1_PARAMETER 		= Overlay0


Temp_PF_Left			= Overlay1
Temp_PF_Right			= Overlay2
Gremlin0_X_Save 		= Overlay1
Gremlin1_X_Save 		= Overlay2
Score_Creditor			= Overlay1

Gremlin0_Y_Temp			= OverlayDouble;ds 1
Gremlin1_Y_Temp			= OverlayDouble+1;Overlay2;ds 1

SWCHA_Current			= Overlay1


SWCHB_Current			ds 1
SWCHB_Last			ds 1


SWCHA_Previous			ds 1




Gremlin_Direction_Counter 	ds 1
;sprite graphics pointers
P0_CurrentRotationSequence 	ds 1; 0-15
P1_CurrentRotationSequence 	ds 1; 0-15



;Playfield Bitmap RAM
Data_PF_Left			ds TOMBSTONE_ROWS
Data_PF_Right			ds TOMBSTONE_ROWS

;these two bytes of RAM are only used inside the kernel itself
;they are set outside the kernel in .Setup_Missile_Pointer
Car0_Y				ds 1
Car1_Y				ds 1

Car0_YFullres			ds 1				; 0..KERNEL_HEIGHT*2-2
Car1_YFullres			ds 1
Car0_Y_Fraction			ds 1
Car1_Y_Fraction			ds 1

Gremlin0_Y			ds 1
Gremlin1_Y			ds 1

;don't move these X variables around.
Car0_X				ds 1
Car1_X				ds 1
Gremlin0_X			ds 1
Gremlin1_X			ds 1
Ball_X				ds 1



Car0_X_Fraction			ds 1
Car1_X_Fraction			ds 1


Car0_X_Acceleration		ds 1
Car1_X_Acceleration		ds 1

Car0_X_Last_Acceleration	ds 1
Car1_X_Last_Acceleration	ds 1

Car0_X_Speed			ds 1
Car1_X_Speed			ds 1

;stores current graphics for forward/reverse
Car0_Gear			ds 1
Car1_Gear			ds 1


Car0_Velocity			ds 1
Car1_Velocity			ds 1


Car0_Pointer			ds 2
Car1_Pointer			ds 2
Gremlin0_Pointer  		ds 2
Gremlin1_Pointer  		ds 2


Gremlin0_ID			ds 1
Gremlin1_ID			ds 1

Gremlin0_Anim_Counter 		ds 1
Gremlin1_Anim_Counter 		ds 1


;these two can share a common byte
Gremlin0_Facing 		ds 1
Gremlin1_Facing 		ds 1


;do something with this
;Gremlin0_SubDirection_A 	ds 1
;Gremlin1_SubDirection_A 	ds 1

;Gremlin0_SubDirection_B 	ds 1
;Gremlin1_SubDirection_B 	ds 1
AUD0_Last_Pitch			ds 1
AUD1_Last_Pitch			ds 1

;this should come from a color lookup table
;that we dynamically read from based on other game state
;each frame ala Indy 500
Border_State 			ds 1
Border_Top_Color 		ds 1
Border_Bottom_Color 		ds 1

Grenade_Column			ds 1

AUD0_Timer			ds 1
AUD1_Timer			ds 1
;these two can probably be merged
Ball_Type			ds 1 ;identifies the mode of the ball
Ball_Size 			ds 1


Ball_Row		 	ds 1
Ball_Control			ds 1		; dual purpose - 
;since bit 0 is insignificant for color, use it to determine mode.
;bit 0 = 1 -> full screen
Forcefield_Generator_Color 	ds 1


;use these in order to "go greyscale" for Color/BW mode
P0_Color			ds 1
P1_Color			ds 1

Timer_Color			ds 1

Background_Color		ds 1

Car0_Score			ds 1;bcd
Car1_Score			ds 1;bcd

Game_Timer			ds 1

Game_State			ds 1; determines when game is running or game over





Car0_Score_Pointer_0X		ds 2
Car0_Score_Pointer_X0		ds 2

Car1_Score_Pointer_0X		ds 2
Car1_Score_Pointer_X0		ds 2


Timer_Pointer_0X		ds 2
Timer_Pointer_X0		ds 2

Seconds_Counter			ds 1




AUD0_Noise_ID			ds 1; determine the next requested noise type
AUD1_Noise_ID			ds 1




;AUD0_Noise_ID_Playing		ds 1; determine the current type of noise triggered
;AUD1_Noise_Playing		ds 1

; CURRENT RAM STATISTICS:

;DESC					BYTES
;---------------------- -------
;PLAYFIELD BITMAP		22 ; big savings due to interleave technique
;OTHER					53

;TOTAL RAM USED			75

;FREE RAM		 	53 ; plenty left for game state, AI, anim/sound



;===============================================================================
; M A C R O S
;===============================================================================

  MAC BIT_B
	.byte	$24
  ENDM

  MAC BIT_W
	.byte	$2c
  ENDM




;===============================================================================
; R O M - C O D E
;===============================================================================

	SEG	Bank0
	ORG	$f000

Start SUBROUTINE
	sei						;			Disable interrupts, if there are any.
	cld						;			Clear BCD math bit.
	ldx	#0
	txs
	pha						;			Set stack to beginning.
	txa
.clearLoop:
	pha
	dex
	bne	.clearLoop


;GAME INIT
;GAME INIT
;GAME INIT
;GAME INIT
;GAME INIT
.Game_Boot ; RUNS ONLY ONCE WHEN THE MACHINE BOOTS UP

	lda	SWCHA
	sta	SWCHA_Previous

	lda	SWCHB
	sta	SWCHB_Last

	lda	#GAME_STATE_INITIAL
	sta	Game_State

	;load immediate or load from a table in a loop??

	lda	#GAME_TIMER_INITIAL
	sta	Game_Timer

	lda	#60
	sta	Seconds_Counter

	lda	#0
	sta	Car0_Score
	sta	Car1_Score
	lda	#0
	sta	Game_State


	lda	#CAR0_GEAR_INDICATOR_REVERSE
	sta	Car0_Gear
	lda	#CAR1_GEAR_INDICATOR_REVERSE
	sta	Car1_Gear


	lda	#BALL_COLUMN_INITIAL
	sta	Grenade_Column



;load in the title graphics
	IF SPLASH = 1

	ldx	#TOMBSTONE_ROWS-1
.titleLoader

	lda	Title_PF_L,X
	sta	Data_PF_Left,X
	lda	Title_PF_R,X
	sta	Data_PF_Right,X 
	dex
	bpl	.titleLoader
	
	ENDIF
;/load in the title graphics



.initialize_animation_counters
	;lda	#GREMLIN_SPEED
	;sta	Gremlin0_Anim_Counter
	;sta	Gremlin1_Anim_Counter
	
	;lda	#1
	;sta	Gremlin1_Direction_Toggle

	;lda	#FACING_RIGHT
	;sta	Gremlin0_Facing

	;lda	#FACING_LEFT
	;sta	Gremlin1_Facing



	jsr	Initialize_Sprites



.InitializeHighBytePointers
	;cars
	lda	#>Car_Rotation_Frame0
	sta	Car0_Pointer+1
	sta	Car1_Pointer+1
	
	;gremlins
	lda	#>Gremlin_Frame0_Left
	sta	Gremlin0_Pointer+1
	sta	Gremlin1_Pointer+1

	;score and timer
	lda	#>Score_0
	sta	Car0_Score_Pointer_0X+1
	sta	Car1_Score_Pointer_0X+1
	sta	Car0_Score_Pointer_X0+1
	sta	Car1_Score_Pointer_X0+1
	sta	Timer_Pointer_X0+1
	sta	Timer_Pointer_0X+1
	








;here we want to have 3 mainloops


.mainLoop:
	jsr	VerticalBlank
	jsr	GameCalc
	jsr	Setup_Player_Missile_Graphics
	jsr	Update_Score_Pointers
	jsr	Draw_Screen
	jsr	OverScan
	
	ldx	#0
	jsr	Sound
	
	ldx	#1
	jsr	Sound
	
	jmp	.mainLoop








Sound SUBROUTINE
	;SOUND PRIORITY
	;1 scream
	;2 crash
	;3 engine/silence (game over)
	;lda 	#NOISE_ID_SCREAM
	;sta	AUD0_Noise_ID


	;first, identify the current noise
	lda	AUD0_Noise_ID,X
.check_scream
	cmp	#NOISE_ID_SCREAM
	beq	.process_scream
.check_crash
	cmp	#NOISE_ID_CRASH
	beq	.process_engine
.check_engine
	cmp	#NOISE_ID_ENGINE
	beq	.process_engine
	
.process_silence

	lda	#0
	sta	AUDV0,X
	rts

.process_scream




	;distortion never changes
	lda 	#4
	sta	AUDC0,X
	

	;this will make sure the screen continues to process over several frames


	lda	AUD0_Timer,X
	sbc	#1
	sta	AUD0_Timer,X
	
	;lda	#0
	beq	.sequence_done

	;jmp	.sequence_done
	
	cmp	#NOISE_DURATION_SCREAM-6;first 15 frames
	bcs	.ramp_up
	
	cmp	#NOISE_DURATION_SCREAM-40
	bcs	.hold

.ramp_down
	;each frame lower pitch by 1

.ramp_down_pitch_start
	LDA	AUD0_Last_Pitch,X
	cmp	#31
	beq	.ramp_down_register
.ramp_down_pitch
	;do scream noise
	adc	#1
	sta	AUD0_Last_Pitch,X
.ramp_down_register

	
	sta	AUDF0,X
	tay
	lda	Scream_Volume_Sweep-7,Y
	sta	AUDV0,X
	rts
.hold

	;hold
	;do scream noise
	lda 	#7;Scream+1,X
	sta	AUD0_Last_Pitch,X
	sta	AUDF0,X
	lda 	#15;Scream+2,X
	sta	AUDV0,X
	rts
	
	;release
.ramp_up



.ramp_up_pitch_start
	LDA	AUD0_Last_Pitch,X
	bne	.ramp_up_pitch
.ramp_up_initialize_pitch
	LDA	#31
	sta	AUD0_Last_Pitch,X
	jmp	.ramp_up_register
.ramp_up_pitch
	;do scream noise
	sbc	#6
	sta	AUD0_Last_Pitch,X
.ramp_up_register
	sta	AUDF0,X
	tay
	lda	Scream_Volume_Sweep-7,Y
	sta	AUDV0,X
	rts

.sequence_done

	LDA	#NOISE_ID_ENGINE
	STA	AUD0_Noise_ID,X
	;noise will go back to engine on next frame
	rts

.process_crash
	rts
.process_engine

	lda	Car0_Velocity,X

	tay
	lda	Engine_Noise_Type,Y
	sta	AUDC0,X
	lda	Engine_Noise_Pitch,Y
	sta	AUDF0,X
	lda	Engine_Noise_Volume,Y
	sta	AUDV0,X
	
	rts
.finish_sound_loop
	rts
	


Initialize_Sprites SUBROUTINE
;INITIALIZE OBJECT POSITIONS
.InitializePositionCars
	lda	#CAR_Y_INITIAL;16		 ;KERNEL_HEIGHT*2-2;16
	sta	Car0_YFullres
	lda	#CAR_Y_INITIAL;16
	sta	Car1_YFullres

	lda	#CAR0_X_INITIAL
	sta	Car0_X
	lda	#CAR1_X_INITIAL
	sta	Car1_X

.InitializePositionGremlins
	lda	#GREMLIN0_Y_INITIAL
	sta	Gremlin0_Y
	lda	#GREMLIN1_Y_INITIAL
	sta	Gremlin1_Y

	lda	#GREMLIN0_X_INITIAL
	sta	Gremlin0_X
	lda	#GREMLIN1_X_INITIAL
	sta	Gremlin1_X
.InitializePositionBall
	lda	#50
	sta	Ball_X
	lda	#6
	sta	Ball_Row

	lda	#1
	sta	VDELBL

;INITIALIZE SPRITE ANIMATION FRAME
.InitializeCarID
	lda	#8
	sta	P1_CurrentRotationSequence
	lda	#0
	sta	P0_CurrentRotationSequence
.InitializeGremlinID
	lda	#1
	sta	Gremlin0_ID
	sta	Gremlin1_ID
	rts


Update_Score_Pointers SUBROUTINE
	;cld
	;extract BCD timer into pointer variables
	lda	Game_Timer
	and	#%00001111
	tax
	lda	Score1_Digit_Table,X
	sta	Timer_Pointer_0X
	
	lda	Game_Timer
	lsr
	lsr
	lsr
	lsr
	tax
	lda	Score1_Digit_Table,X
	sta	Timer_Pointer_X0



	;CAR0 SCORE
	lda	Car0_Score
	and	#%00001111
	tax
	lda	Score0_Digit_Table,X
	sta	Car0_Score_Pointer_0X
	
	lda	Car0_Score
	lsr
	lsr
	lsr
	lsr
	tax
	lda	Score0_Digit_Table,X
	sta	Car0_Score_Pointer_X0
	
	
	
	;CAR1 SCORE
	lda	Car1_Score
	and	#%00001111
	tax
	lda	Score1_Digit_Table,X
	sta	Car1_Score_Pointer_0X

	lda	Car1_Score
	lsr
	lsr
	lsr
	lsr
	tax

	lda	Score0_Digit_Table,X
	sta	Car1_Score_Pointer_X0

	
	rts

VerticalBlank SUBROUTINE
	lda	#2
	sta	WSYNC
	sta	VSYNC
	sta	WSYNC
	sta	WSYNC
	lsr
	ldx	#44
	sta	WSYNC
	sta	VSYNC
	stx	TIM64T
	
	
	;build stuff in here??
	
	
	rts
	



Stagger_Step_Animation SUBROUTINE

	rts





Game_Reset SUBROUTINE
	;lda	#$4b
	;sta	COLUBK

	lda	#GAME_STATE_START
	sta	Game_State

	lda	#60
	sta	Seconds_Counter

	lda	#1
	sta	AUD0_Noise_ID
	STA	AUD1_Noise_ID


;.Update_Score_Pointers
;	jsr	Update_Score_Pointers
  

;.clear title graphics or any leftover tombstones

	lda	#0
	ldx	#TOMBSTONE_ROWS-1
.Clear_Tombstones
	sta	Data_PF_Left,X
	sta	Data_PF_Right,X 
	dex
	bpl	.Clear_Tombstones



.Reset_Timer
	lda	#GAME_TIMER_START
	sta	Game_Timer

.Reset_Score
	lda	#0
	sta	Car0_Score
	
	sta	Car1_Score	

	JSR	Initialize_Sprites

	rts
	
; this has to branch off depending on game mode
GameCalc SUBROUTINE



	inc	Frame_Counter
	lda	SWCHB
	and	%00000001
	sta	SWCHB_Current
	
	;IF current state eq 1 (reset not depressed)
	bne	.bind_ON_RESET_TOGGLE;we have to check for ON_RESET_TOGGLE event

	;ELSE if current state eq 0 	(reset depressed)
		;we do NOT have to check for ON_RESET_TOGGLE event

	
	lda	#0
	sta	Game_State;freeze the game
	
	jmp	.finish_timer; just skip ahead

;figure out if this is the ON_RESET_DOWN event or not

;IF (current state eq 0 ...			(reset not depressed)
.bind_ON_RESET_TOGGLE

	lda	SWCHB_Last
	and	%00000001
	;...and	previous state eq 0) 		(reset previously depressed) 
	beq	.ON_RESET_TOGGLE; THEN perform game reset
 
	;ELSE just skip ahead

	jmp	.skip_game_reset; just skip ahead
	
.ON_RESET_TOGGLE; previous was 1 and	current is 0 so do reset

	jsr	Game_Reset
	jmp	.finish_timer
	
.skip_game_reset
	;inc	Frame_Counter

	lda	Game_State
	beq	.finish_timer ;if game_state = 0 (game over) then do not decrement timer


	lda	Frame_Counter
	dec	Seconds_Counter
	bne	.finish_timer



.Decrement_Timer
	lda	#TICK_LENGTH ; reset counter
	sta	Seconds_Counter


	
	sed
	lda	Game_Timer
	sbc	#$0;I don't know why, but whatever number I put in here it subtracts that number +1
	cld
	sta	Game_Timer


	
	

	beq	.end_Game
	;jsr	Update_Score_Pointers
	jmp	.finish_timer
	
.end_Game
	;jsr	Update_Score_Pointers
	lda	#GAME_STATE_OVER; end game due to timer running out
	sta	Game_State
	
	lda	#0
	sta	AUD0_Noise_ID
	STA	AUD1_Noise_ID
.finish_timer
	lda	SWCHB_Current
	sta	SWCHB_Last



;this block should always execute except when colors are cycling
.SetColors

	lda	Game_State
	beq	.Game_Over_Colors

	lda	#BACKGROUND_COLOR; set to black
	sta	Background_Color
	jmp	.continue_SetColors

.Game_Over_Colors
	lda	#BACKGROUND_COLOR_GAME_OVER; set to grey
	sta	Background_Color
	
.continue_SetColors

	;finish the rest of the score graphics later
	lda	SWCHB
	and	%00001000
	beq	.SetColors_Black_and_White
	;COLOR
	
	;BORDER
	lda	#BORDER_STATE_INITIAL
	sta	Border_State

	;set border colors based on Border_State
	ldx	Border_State
	lda	Border_Type_Table,X
	sta	Border_Top_Color
	sta	Border_Bottom_Color
	
	
	lda	#FORCEFIELD_GENERATOR_COLOR_C
	sta	Forcefield_Generator_Color
	
	;GRENADE
	ldx	Ball_Type
	lda	Grenade_Color_Table,X
	sta	Ball_Control
	
	;TIMER
	lda	#TIMER_COLOR_C
	sta	Timer_Color
	
	;P0/P1
	lda	#CAR0_COLOR_C
	sta	COLUP0
	sta	P0_Color; this is only used in the score display
	lda	#CAR1_COLOR_C
	sta	COLUP1
	sta	P1_Color; this is only used in the score display
	
	jmp	.SetColors_Done

	;B&W
.SetColors_Black_and_White

	;BORDER
	ldx	Border_State
	lda	Border_Type_Table+3,X
	sta	Border_Top_Color
	sta	Border_Bottom_Color
	
	lda	#FORCEFIELD_GENERATOR_COLOR_BW
	sta	Forcefield_Generator_Color
	
	;GRENADE
	ldx	Ball_Type
	lda	Grenade_Color_Table+4,X
	sta	Ball_Control

	;TIMER
	lda	#TIMER_COLOR_BW
	sta	Timer_Color

	
	;P0/P1
	lda	#CAR0_COLOR_BW
	sta	COLUP0
	sta	P0_Color
	lda	#CAR1_COLOR_BW
	sta	COLUP1
	sta	P1_Color





.SetColors_Done


;set ball config based on Ball_Type variable
	ldx	Ball_Type
	cpx	#3
	bne	.ClearBarrier
.SetBarrier
	lda	#1
  ; brk
	;ora Ball_Control
	sta	Ball_Control
.ClearBarrier
	lda	Ball_Control
	and	%11111110
	sta	Ball_Control


	ldx	Ball_Type
	lda	Grenade_Size_Table,X
	sta	Ball_Size

	
.skipRandom:

; move the ball:
	lda	Frame_Counter
	and	#$03
	bne	.skipMoveBL

	ldx	Ball_X
	dex
	cpx	#17
	bcs	.okBall_X
	ldx	#143
.okBall_X:
	stx	Ball_X
.skipMoveBL:



;this chunk should always execute regardless of game state


	ldx	#2
;update zombie walkcycles
.walkCycler

;first, update counters and	determine whether or not to update
;the walk cycle on this frame
	dec	Gremlin0_Anim_Counter-1,X
	bne	.exitWalkCycle ; don't update until timer reaches 0
.walkCycle1Reset
	lda	#GREMLIN_SPEED; this should be based on zombie speed, variable
	sta	Gremlin0_Anim_Counter-1,X

.animate_zombie
	;lda 	Game_State
	;jsr	Stagger_Step_Animation

	;jsr	moveGremlin1
	dec	Gremlin_Direction_Counter
	dec	Gremlin0_ID-1,X
	bne	.exitWalkCycle
.walkCycle1Repeat
	lda	#GREMLIN_FRAMES
	sta	Gremlin0_ID-1,X

.exitWalkCycle
	dex
	bne	.walkCycler

	rts


Setup_Sprite_Pointer SUBROUTINE
.Setup_Missile_Pointer	
	lda	Car0_YFullres,X
	sta	VDELP0,X
	lsr	;divide X by two
	sta	Car0_Y,X

	lda	P0_CurrentRotationSequence,X
	tay
	
	lda	Car_Rotation_Sequence,Y; find the proper frame depending on direction
	clc
	adc	Car0_Y,X; add offset to Y axis
	

	cpx	#0
	bne	.sta_Car1_Pointer
	sta	Car0_Pointer
	jmp	.Setup_Player_Pointer
.sta_Car1_Pointer
	sta	Car1_Pointer

	
.Setup_Player_Pointer
; calculate pointer to missile #0 graphics:
	lda	Gremlin0_Y,X
	and	#~$80
	clc
	adc	#GREMLIN_HEIGHT-1
	sta	Gremlin0_Y_Temp,X
	tay

	lda	Gremlin0_ID,X
	adc	Gremlin0_Facing,X
	
	stx	Temp_Var;save		X
	
	tax
	lda	Gremlin_Table,X
	
	ldx	Temp_Var;restore	X
	sec
	sbc	Gremlin0_Y_Temp,X
	
	
	;fixed?
	cpx	#0
	bne	.sta_Gremlin1_Pointer
	sta	Gremlin0_Pointer; 2 byte equate
	jmp	.setcontinue
.sta_Gremlin1_Pointer
	sta	Gremlin1_Pointer; 2 byte equate
.setcontinue
	;/fixed?


	;fix
	cpy	#KERNEL_HEIGHT-1
	bmi	 .skipWrap0
	ldy	#KERNEL_HEIGHT
	;/end Fix
	
	
.skipWrap0:
	cpx	#0
	bne	.lda_Gremlin1_Pointer
	lda	(Gremlin0_Pointer),Y; 2 byte equate
	jmp	.setcontinue2
.lda_Gremlin1_Pointer
	lda	(Gremlin1_Pointer),Y; 2 byte equate
.setcontinue2
	asl
	asl 
	sta	NUSIZ0,X
	RTS

	
Setup_Player_Missile_Graphics SUBROUTINE
; *** calculate kernel variables: ***
; calculate pointer to Car #0 graphics:
; optimized with the above sub subroutine on 1/6/04
; there should be a way to put below into a loop.  I don't know how yet
; without using up more ROM than what below requires

	ldx	#1

	JSR	Setup_Sprite_Pointer


	ldx	#0
	JSR	Setup_Sprite_Pointer

	
;BEGIN PRECOMPENSATION

;The missiles rely on the cumulative effect of relative position changes.
;When the missiles are at the top of the screen, however, their starting horizontal
;position needs to be absolutely defined.  So what we have to do is add up
;all the relative changes up to the first visible missile row and	apply this
;as precompensation before the screen starts.  That way the horizontal position
;will match what it would be if the zombie were not at the top of the screen.

;the first thing we need to do is figure out which zombie may be at the top of the screen.

;then we need to figure out how many rows of the zombie are being cut off.
;we then take this number and	loop over it, aggregating the HMMx values
;as we go along.  This simulates what the kernel would have done had the
;zombie been completely in the screen.  The final aggregate value is converted
;back into the proper HMMx constant and	stored in HMMx and	applied with an HMOVE.

	;save off Gremlin X position
	lda	Gremlin1_X
	sta	Gremlin1_X_Save
	
	lda	Gremlin0_X
	sta	Gremlin0_X_Save
	;/save off Gremlin X position


;loop through this 8 times

;FROM: KERNEL_HEIGHT-1+GREMLIN_HEIGHT (maximum extent of gremlin, off screenn at top)
;TO: KERNEL_HEIGHT-1 (minimum extent of gremlin, collision with top)

	ldy	#KERNEL_HEIGHT-1+GREMLIN_HEIGHT
.do_adjustments
	jsr	HMMx_Aggregate_Line; this aggregates the offsets
	dey
	cpy	#KERNEL_HEIGHT-1
	bpl	.do_adjustments



; perform initial horizontal positioning updates for all objects
	ldx	#4
.loopPos:
	lda	Car0_X,X
	jsr	SetPosX
	dex
	bpl	.loopPos
	sta	WSYNC

	sta	HMOVE
	sta	WSYNC

	sta	HMCLR
	
	;restore Gremlin X position
	lda	Gremlin1_X_Save
	sta	Gremlin1_X
	
	lda	Gremlin0_X_Save
	sta	Gremlin0_X 
	;/restore Gremlin X position

	rts


;LOAD Y with SCANLINE BEFORE CALLING THIS FUNCTION
HMMx_Aggregate_Line SUBROUTINE

	
.isM0atTop 
; determine whether to draw Gremlin #0:
	cpy	Gremlin0_Y_Temp
	bpl	.Gremlin1_Compensation_Start; no gremlin1, so go to g0 routine
	;.drawM1:

	iny ; this is necessary because of the interleaved nature of M0 vs. M1
	lda	(Gremlin0_Pointer),Y
	dey ; this is necessary because of the interleaved nature of M0 vs. M1
	tax
	
	;this is really only necessary on the top line
	;this is wasting a fair amount of cycles
	asl
	asl
	sta	NUSIZ0
	txa
	lsr
	lsr
	lsr
	lsr
	sta	Gremlin_Pointer_Temp
	clc
;	if the offset is less than 12, then it's either zero or a positive offset
	cmp	#12
	bcc	.Gremlin0_Compensation_Positive_to_Negative
.Gremlin0_Compensation_Negative_to_Positive
	;brk
	sbc	#12; brings the numbers back down to a 0-3 range
	tax
	lda	HMMx_Compensation_Table,X; look up the conversion index based on the table
	adc	Gremlin0_X; Gremlin1_X = Gremlin1_X + A
	sta	Gremlin0_X
	jmp	.Gremlin1_Compensation_Start
.Gremlin0_Compensation_Positive_to_Negative
	;just use the accumulator as is to subtract from Gremlin1_X
	lda	Gremlin0_X
	adc	#1; I don't know why this hack is necessary
	sbc	Gremlin_Pointer_Temp; Gremlin1_X = Gremlin1_X - A
	sta	Gremlin0_X 


.Gremlin1_Compensation_Start


; determine whether to draw Gremlin #1:
	ldx	Gremlin1_Y_Temp
	cpy	Gremlin1_Y_Temp
	bpl	.Gremlin1_Compensation_End; no gremlin1
	;.drawM1:
	
	;this dey/iny thing is a hack.
	;dey
	lda	(Gremlin1_Pointer),Y
	;iny
	tax
	
	;this is really only necessary on the top line
	;this is wasting a fair amount of cycles
	asl
	asl
	;this might be the wrong value
	sta	NUSIZ1
	txa
	lsr
	lsr
	lsr
	lsr
	sta	Gremlin_Pointer_Temp
	clc
;	if the offset is less than 12, then it's either zero or a positive offset
	cmp	#12
	bcc	.Gremlin1_Compensation_Positive_to_Negative
.Gremlin1_Compensation_Negative_to_Positive
	;brk
	sbc	#12; brings the numbers back down to a 0-3 range
	tax
	lda	HMMx_Compensation_Table,X; look up the conversion index based on the table
	adc	Gremlin1_X; Gremlin1_X = Gremlin1_X + A
	sta	Gremlin1_X
	rts
.Gremlin1_Compensation_Positive_to_Negative
	;just use the accumulator as is to subtract from Gremlin1_X
	lda	Gremlin1_X
	adc	#1; I don't know why this hack is necessary
	sbc	Gremlin_Pointer_Temp; Gremlin1_X = Gremlin1_X - A
	sta	Gremlin1_X 

	
.Gremlin1_Compensation_End
;END PRECOMPENSATION

	rts
;/HMMx_Aggregate_Line SUBROUTINE




	
NextRandom SUBROUTINE
;	lda	Random			 		; 3
;	lsr						; 2
;	bcc	.skipEor				; 2
;	eor	#RAND_EOR_8				; 2
;.skipEor:						;	= 11/12 (~11.5)
;	sta	Random			 		; 3

	lda	Random					; 3
	asl						; 2
	eor	Random					; 3
	asl						; 2
	eor	Random					; 3
	asl						; 2
	asl						; 2
	eor	Random					; 3
	asl						; 2
	rol	Random					; 5
	rts						; 6
;NextRandom END


	align 256

Draw_Screen	SUBROUTINE





	;player color playfield
	lda	Ball_Size
	ora	#%00010010; important
	sta	CTRLPF

	lda	Forcefield_Generator_Color
	sta	COLUPF

	
	; lda	#$02  ; use this for a custom background color for the score
	; sta	COLUBK

; setting up a timer to make experimenting inside the kernel easier
; (the final code shouldm't need this):
	ldx	#228
.waitTim:
	lda	INTIM
	bne	.waitTim
	sta	WSYNC
	sta	VBLANK
	stx	TIM64T


	ldx	#0
	ldy	#SCORE_HEIGHT
	

	
	STA	WSYNC
	

	





;This whole score section required some very careful timing, but naturally, I did
;it mostly though trial and	error, rather than calculating the cycle times. :)

	sleep 4
.Draw_Score
;	------------------------------------  scanline 1 (SCORE)
	;SLEEP 4
	stx.w	PF0;blank out PF0, or set it to GEAR graphic (on next line when it loops back)
	stx	PF2
	lda	(Car0_Score_Pointer_X0),Y; CAR0 - TENS DIGIT
	and	#%11100000
	sta	PF1 ; get the write over with ASAP to buy us time
	lda	(Car0_Score_Pointer_0X),Y; CAR0 - ONES DIGIT
	and	#%00001111
	sta	PF1
 
	lda	P1_Color
	STA	COLUP1	

	
	lda	(Car1_Score_Pointer_X0),Y; CAR1 - TENS DIGIT
	and	#%00001111
	lsr
	sta	PF1
	lda	(Car1_Score_Pointer_0X),Y; CAR1 - ONES DIGIT
	and	#%00001111
	sta.w	PF2

	stx	PF0
	stx	PF1
	stx	PF2


	;normal playfield


	
	SLEEP 2
;	------------------------------------   scanline 2 (TIMER AND GEAR INDICATORS)

	
	lda	Car0_Gear;Car0_Gear; the gear value is also used as the graphics
	;lda	#$FF
	sta	PF0

	;toggle in and	out of PF modes


	;wedge in a color color gradient for the gear indicator and score
	;tya
	lda	Score_Gradient-1,Y	
	ora	Timer_Color


	STA	COLUP0	
	STA	COLUP1
	
	SLEEP 8

	lda	(Timer_Pointer_X0),Y;TIMER - TENS DIGIT
	and	#%11110000
	sta	PF2

	;INSERT TIMER CODE HERE
	lda	(Timer_Pointer_0X),Y;TIMER - ONES DIGIT
	and	#%11110000
	
	sta	PF0; write in PF0

	
	SLEEP 4
	
	
	
	lda	P0_Color
	STA	COLUP0	

	SLEEP 12
	lda	Car1_Gear;Car1_Gear; gear indicator
	sta	PF2

	
	sta	WSYNC ; end of scanline pair

	dey
	bne	.Draw_Score
;/.Draw_Score


	lda	P1_Color
	STA	COLUP1	

	
	;lda	Background_Color
	;sta	COLUBK
	sty	PF0;y = 0
	sty	PF1
	sty	PF2

; reset playfield/ball settings
	lda	Ball_Size
	ora	#1
  ;  and	Ball_Size; important
	sta	CTRLPF
	ldx	Border_Top_Color
	lda	#$FF; top border
	sta	WSYNC


	stx	COLUPF
	sta	PF0
	sta	PF1
	sta	PF2


; KERNEL MAIN (TRY NOT TO ALTER THIS)
	include dd_kernel_main.asm
	;this ends with a JTS
;/KERNEL MAIN


ExitKernel:
	ldx	#$FF
	lda	#0
	ldy	Border_Bottom_Color
	sta	WSYNC
	sta	ENAM0
	sta	ENAM1
	sta	GRP0
	sta	GRP1
	sta	GRP0; this is necessary due to VDEL
	
;---------------------------------------
	sty	COLUPF
	stx	PF0			;					already 0
	stx	PF1
	stx	PF2
	ldx	#0
	sta	WSYNC; do 2nd line of border
	ldx	Forcefield_Generator_Color
  ; sta	WSYNC
	sta	PF0
	sta	PF1
	sta	PF2
	stx	COLUPF
	sta	WSYNC
	ldx	#2
.waitScreen0:
	lda	INTIM
	bne	.waitScreen0
	sta	WSYNC
	stx	VBLANK
	rts










	




;DRIVING CONTROLLER CODE
Rotation_Processor  SUBROUTINE

.Read_Driving_Controller
	lda	SWCHA_Previous
	jsr	Load_Nybble
	and	#%00000011
	tay	;copy the result to Y
	
	
	lda	SWCHA_Current

	
	jsr	Load_Nybble

	and	#%00000011
	eor	Next_DC_Table,Y  ; works like cmp	for beq
	beq	.leftTurn
	eor	#%00000011
	beq	.rightTurn
	RTS	;no need to do further processing because there is no change to rotation
.leftTurn:
	dec	P0_CurrentRotationSequence,X
	JMP	.Rotation_Wrap
.rightTurn:
	inc	P0_CurrentRotationSequence,X


;check boundaries
.Rotation_Wrap
	lda	P0_CurrentRotationSequence,X
	cmp	#$FF
	beq	.Rotation_Wrap_Left
	cmp	#16
	beq	.Rotation_Wrap_Right
	jmp	.Player_Reflect_Check
.Rotation_Wrap_Left
	lda	#15
	sta	P0_CurrentRotationSequence,X
	jmp	.Player_Reflect_Check
.Rotation_Wrap_Right
	lda	#0
	sta	P0_CurrentRotationSequence,X


;set reflect bit accordingly
.Player_Reflect_Check
	;this will have been modified by the controller routines
	ldy	#TurnOffSpriteReflect; initialize to no flip
	
	;this will load either P0 or P1 depending on (X) P0_P1_PARAMETER
	lda	P0_CurrentRotationSequence,X;0-15
	cmp	#FlipThreshold
	bmi	.Continue0	; if A < 9 then sigN will be 1
			; so do not flip, just .Continue
.FlipSprite0
	ldy	#TurnOnSpriteReflect

.Continue0
	sty	REFP0,X
	RTS



;/DRIVING CONTROLLER CODE



;HANDLE FORWARD/REVERSE GEAR SWITCHING


Transmission SUBROUTINE; Y cable joystick hack
	stx	Car_Picker
	
	lda	SWCHA_Current
	jsr     Load_Nybble
	
	and	#%00001100;only look at the joystick bits for left and right, not the driving controller bits
	cmp	#%00000100
	beq	.CarReverseGear
	cmp	#%00001000
	beq	.CarForwardGear
	JMP	.Gas_Pedal
	;bcs	.Car1Transmission
.CarForwardGear

	lda	#0
	sta	Car0_Gear,X
	JMP	.Gas_Pedal
.CarReverseGear

	lda	CarGearGFX,X
	sta	Car0_Gear,X
	;RTS

;/HANDLE FORWARD/REVERSE GEAR SWITCHING

	
.Gas_Pedal
	;do gas pedal
	;ldx	Car_Picker
	txa
	;read the fire button
	beq	.other_button
	lda	INPT5;INPT,X
	bpl	.Accelerate
	jmp     .Decelerate

.other_button
	lda	INPT4;INPT,X


	bpl	.Accelerate
.Decelerate
	lda	#0
	sta	Car0_Velocity,X
	rts
	
.Accelerate 
	lda	#1
	sta	Car0_Velocity,X

	;INC Car0_X

.Process_X_Delta

	ldx	Car_Picker
	lda	P0_CurrentRotationSequence,X
	;if the direction is greater than 8, then flag the potential X delta as negative
	tay

	cmp	#8
	bcc	.delta_positive;>= 8	
.delta_negative
	lda	#1
	sta	Overlay2;flag for negative
	JMP	.continue_Accelerate
.delta_positive
	lda	#0
	sta	Overlay2;flag for positive


.continue_Accelerate 

	;look up fractional amount for X

	lda	Car0_X_Fraction,X		; load the current fractional value for the car
	clc
	adc 	X_Fraction_Table,Y		; add the proper amount of X delta to the fraction
	
	sta	Car0_X_Fraction,X		;save it for later
		
	bcs	.X_Plus_or_Minus_1		; if carry set, then add a pixel
	JMP	.Process_Y_Delta		; otherwise skip ahead and process Y
.X_Plus_or_Minus_1
	;figure out if we should apply positive or negative movement
	;first, determine native movement (assuming forward gear)

	
	;flip the delta bit depending on gear status
	LDA	Car0_Gear,X
	AND	#%00000001;isolate bit 0
	
	EOR	Overlay2;flip the delta
	
	
	
	beq	.X_Minus_1
.X_Plus_1
	INC	Car0_X,X		;move the car 1 pixel to the right
	JMP	.Process_Y_Delta
.X_Minus_1
	DEC	Car0_X,X		;move the car 1 pixel to the left



;Y MOVEMENT FOR CARS
;Y MOVEMENT FOR CARS
;Y MOVEMENT FOR CARS
;Y MOVEMENT FOR CARS
;Y MOVEMENT FOR CARS

.Process_Y_Delta

	;ldx	Car_Picker
	lda	P0_CurrentRotationSequence,X
	
	
	;if the direction is greater than 8, then flag the potential X delta as negative
	tay
	;if it's under 5, then add 8
	
	cmp	#5
	;RTS
	bcs	.no_offset
	
	JMP	.delta_Y_negative
	
	
	;adc	#12
	
.no_offset
	sbc	#4
	
	cmp	#8
	bcc	.delta_Y_positive;.if it's less than 4
.delta_Y_negative
	lda	#1
	sta	Overlay1;flag for negative
	JMP	.continue_Y_Accelerate
.delta_Y_positive
	lda	#0
	sta	Overlay1;flag for positive


.continue_Y_Accelerate 

	;look up fractional amount for X

	lda	Car0_Y_Fraction,X		; load the current fractional value for the car
	clc
	adc 	Y_Fraction_Table,Y		; add the proper amount of X delta to the fraction
	sta	Car0_Y_Fraction,X		;save it for later
	bcs	.Y_Plus_or_Minus_1		; if carry set, then add a pixel
	rts					; otherwise done
.Y_Plus_or_Minus_1
	;figure out if we should apply positive or negative movement
	;first, determine native movement (assuming forward gear)

	
	
	;flip the delta bit depending on gear status
	LDA	Car0_Gear,X
	AND	#%00000001;isolate bit 0
	
	EOR	Overlay1;flip the delta
	
	
	
	beq	.Y_Minus_1
.Y_Plus_1
	INC	Car0_YFullres,X		;move the car 1 pixel up
	rts
.Y_Minus_1
	DEC	Car0_YFullres,X		;move the car 1 pixel down
	rts



;PHYSICS ATTRIBUTES:

;VARIABLE			NAME			DESCRIPTION
;==============================	======================= ======================================
;DIRECTION 			DIRECTION		WHICH DIRECTION THE CAR IS POINTING
;Car0_Gear 			GEAR 			FORWARD OR REVERSE GEAR
;Car0_RPM			RPM			ENGINE SPEED
;Car0_CURRENT_DIRECTION 	CURRENT_DIRECTION	WHICH DIRECTION THE CAR IS CURRENTLY MOVING
;Car0_CURRENT_SPEED 		CURRENT_SPEED		HOW FAST THE CAR IS CURRENTLY MOVING

;COUNTERS
;Car0_

;two RAM counters can be used to do simple accelleration.
;the first variable is a delay inbetween speeds.
;the second variable is a speed indicator.

;store how long it takes to accelerate from speed to speed






OverScan SUBROUTINE
	lda	#36
	sta	TIM64T

	bit	SWCHB
	bmi	.set_Barrier; difficulty switch was toggled
	lda	#0; powerup
	jmp	.store_Ball_type
.set_Barrier
	lda	#3; barrier
.store_Ball_type
	sta	Ball_Type

;don't both with collision detection if the game is over

	lda 	Game_State
	beq 	.skip_collision_detection

;COLLISION DETECTION

	;this indexer is based on missile, not player
	ldx	#1
.collision_detection

	lda	CXM0P,X; check the missile for collisions
	beq	.no_killings;this missile was not hit
	
	;current missile was hit.  now figure out which player hit it


	;	7	6
	;CXM0P 	M0 P1 	M0 P0		0
	;CXM1P 	M1 P0 	M1 P1		1

 	
 	;.M0_P1
 	;award point to P1
 
 	;.M0_P0
 	;award point to P1
 	
 	;.M0
 	
 	;.M1_P0
 	;award point to P0
 	
 	;.M1_P1
	;award point to P0


	txa
	eor	#1
	sta	Score_Creditor





	
	;the collision state shouldn't exceed one frame
	;so take this out ultimately.  The collision is going to change to a collision
	;with the resultant tombstone which shouldn't generate a crash sound
	


	;see if we're already processing a scream on this audio channel
	LDA	AUD0_Noise_ID,X
	cmp	#NOISE_ID_SCREAM
	beq	.skip_flag_scream
.flag_scream

	;flag the noise for scream if it isn't already playing
	;lda	#NOISE_ID_SCREAM
	;sta	AUD0_Noise_ID,X
	lda 	#NOISE_ID_SCREAM
	sta	AUD0_Noise_ID,X
	
	lda	#NOISE_DURATION_SCREAM	;set the duration for the sound
	sta	AUD0_Timer,X		;initialize the timer



	stx	Temp_Var
	ldx	Score_Creditor
	;score increment (only do it if the scream expires)
	lda	Car0_Score,X
	

	sed
	clc
	adc	#1
	cld
	sta	Car0_Score,X
	
	ldx	Temp_Var
	
	;reposition gremlin that was hit
	lda	#0
	sta	Gremlin0_X,X

	;ADD RANDOMIZATION ON THE Y ROUTINE
	

.skip_flag_scream			;scream already in progress
	


	
	
	
.no_killings
 	dex
 	bpl	.collision_detection
 
.skip_collision_detection
	sta	CXCLR
;BEGIN USER INPUT SEGMENT
;BEGIN USER INPUT SEGMENT
;BEGIN USER INPUT SEGMENT
;BEGIN USER INPUT SEGMENT

	;if game is over, freeze the cars wherever they last were
	lda 	Game_State
	beq 	.skip_Car_Engine
	
	ldx	#2 ; debugging
.Read_Controllers
	;this should be the only sample of SWCHA for the frame
	lda	SWCHA
	sta	SWCHA_Current


	dex 
	stx	Temp_Var

	jsr	Rotation_Processor
	
	cpx	#0
	bne	.nodo
	lda	SWCHA_Current;
	sta	SWCHA_Previous; save this for next time, previous is now in Y
.nodo	
	jsr	Transmission


	ldx	Temp_Var
	bne	.Read_Controllers



.skip_Car_Engine
	rts


;A=	SOURCE BYTE/RESULT NYBBLE (aligned right)
;X=1	RIGHT NYBBLE
;x=0	LEFT  NYBBLE
Load_Nybble SUBROUTINE
	cpx	#1
	beq	.no_shifting
.shifter
	LSR
	LSR
	LSR
	LSR
.no_shifting
	rts



;GREMLIN ANIMATION
; turn this into a bounds-checking-only routine
Bounds_Check_Gremlins SUBROUTINE
	rts	; debugging

	ldx	#1
	
.loopBounds_Check_Gremlins:

	;BOUNDS CHECK ON Y
	lda	Gremlin0_Y,X
	
	
	cmp	#KERNEL_HEIGHT+1|$80	;gremlin Y limit (bottom)
	ldy	#1|$80			;gremlin Y limit (top)

	
	
	cpy	#0|$80			;gremlin Y limit (top)
	ldy	#KERNEL_HEIGHT|$80	;gremlin Y limit (bottom)



	sta	Gremlin0_Y,X
	


	;BOUNDS CHECK ON X
	cmp	#SCREEN_WIDTH;gremlin X limit (right)
	ldy	#0
	
	


	cmp	#0	
	ldy	#0
	

	sta	Gremlin0_X,X

	dex
	bpl	.loopBounds_Check_Gremlins

	rts


SetPosX SUBROUTINE
; TODO: use new Fatal Run method discovered by Manuel
; calculates the values and	positions objects:
; Input:
; - a = x-position
	tay					; 2
	iny					; 2
	tya				 	; 2
	and	#$0F			 	; 2
	sta	Temp_Var		 	; 3
	tya					; 2
	lsr					; 2
	lsr					; 2
	lsr					; 2
	lsr					; 2
	tay					; 2
	clc					; 2
	adc	Temp_Var			; 3
	cmp	#$0F			 	; 2
	bcc	.skipIny		 	; 2
	sbc	#$0F			 	; 2
	iny					; 2
.skipIny:
	sta	WSYNC				; 3
	eor	#$07			 	; 2
	asl					; 2
	asl					; 2
	asl					; 2
	asl					; 2
	sta.wx	HMP0,X				; 5		@15
.waitPos:
	dey					; 2
	bpl	.waitPos		 	; 2
.bankCheck:
	sta	RESP0,X				; 4
	rts					; 6
;SetPosX END


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
DATA_START = $FE00

	org	DATA_START

;DATA
CarGearGFX
	.byte CAR0_GEAR_INDICATOR_FORWARD
	.byte CAR1_GEAR_INDICATOR_FORWARD
	
	;IF SPLASH = 1

Title_PF_L:  ;	76543210
	.byte %00000000 | %00000000 ;%00000000
	.byte %00000000 | %00000000
	.byte %00000000 | %00000000
	.byte %00000000 | %00101111
	.byte %00011000 | %00000000
	.byte %00011000 | %00000000
	.byte %00100100 | %00000000
	.byte %01000010 | %00000000
	.byte %10000001 | %00000000
	.byte %01000000 | %00011111
	.byte %00000000 | %00000000

	.byte 0
Title_PF_R:  ;	01234567

	.byte %00000000 | %00000000
	.byte %00000000 | %00000000
	.byte %01010111 | %00000000
	.byte %00000100 | %00100000
	.byte %00001000 | %00010000
	.byte %00010000 | %00001000
	.byte %00100000 | %00000100
	.byte %01000000 | %00000010
	.byte %00111111 | %00000000
	.byte %00000000 | %00000000
	.byte %00000000 | %00000000  
	
	.byte 0
	
	;END IF

	
Car_Rotation_Sequence:
	;NO FLIP BIT - FACE RIGHT
	.byte <Car_Rotation_Frame0 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;0  - point up
	.byte <Car_Rotation_Frame1 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;1  - 
	.byte <Car_Rotation_Frame2 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;2  - 
	.byte <Car_Rotation_Frame3 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;3  - 
	.byte <Car_Rotation_Frame4 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;4  - point right
	.byte <Car_Rotation_Frame5 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;5  - 
	.byte <Car_Rotation_Frame6 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;6  - 
	.byte <Car_Rotation_Frame7 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;7  - 
	.byte <Car_Rotation_Frame8 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;8  - point down
	; WITH FlipThreshold - FACE LEFT
	.byte <Car_Rotation_Frame7 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;9  - 
	.byte <Car_Rotation_Frame6 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;10 - 
	.byte <Car_Rotation_Frame5 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;11 - 
	.byte <Car_Rotation_Frame4 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;12 - point left
	.byte <Car_Rotation_Frame3 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;13 - 
	.byte <Car_Rotation_Frame2 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;14 - 
	.byte <Car_Rotation_Frame1 - TOMBSTONE_ROWS*TOMBSTONE_HEIGHT - CAR_HEIGHT;15 - 
	

Gremlin_Table:
	.byte	0;dummy byte
	.byte	<Gremlin_Frame0_Left  + GREMLIN_HEIGHT-1; points to top of graphics
	.byte	<Gremlin_Frame1_Left  + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame2_Left  + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame3_Left  + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame4_Left  + GREMLIN_HEIGHT-1
	
	.byte	<Gremlin_Frame0_Right + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame1_Right + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame2_Right + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame3_Right + GREMLIN_HEIGHT-1
	.byte	<Gremlin_Frame4_Right + GREMLIN_HEIGHT-1



; I don't know why these have to be -1 less but they do
HMMx_Compensation_Table:
;neg to pos
	.byte	3
	.byte	2
	.byte	1
	.byte	0

Pot2Tbl:
V SET 1
  REPEAT	8
	.byte V
V SET V + V
  REPEND
;	.byte	$01, $02, $04, $08, $10, $20, $40, $80

MaskTbl:
V SET %01010101
  REPEAT	TOMBSTONE_ROWS
	.byte V
V SET V ^ $ff
  REPEND

	org	DATA_START + TOMBSTONE_ROWS*TOMBSTONE_HEIGHT + CAR_HEIGHT

Car_Rotation_Frame0:;UP (ACTUAL)
  ;.byte %00000000
	.byte %01011010; .X.XX.X.
	.byte %01111110; .XXXXXX.
	.byte %01100110; .XX..XX.
	.byte %00011000; ...XX...
	.byte %01011010; .X.XX.X.
	.byte %01111110; .XXXXXX.
	.byte %01011010; .X.XX.X.
	.byte %00011000; ...XX...

Car_Rotation_Frame1:
  ;.byte %00000000
  
	.byte %00001100; ....XX..
  	.byte %00111100; ..XXXX..
	.byte %11101000; XXX.X...
	.byte %11011010; XX.XX.X.
  	.byte %00011010; ...XX.X.
  	.byte %01011111; .X.XXXXX
  	.byte %00111100; ..XXXX..
	.byte %00101100; ..X.XX..


Car_Rotation_Frame2:
  ;.byte %00000000
	.byte %00011000; ...XX...
	.byte %01111000; .XXXX...
	.byte %01000010; .X....X.
	.byte %11011011; XX.XX.XX
	.byte %11011100; XX.XXX..
	.byte %00011110; ...XXXX.
	.byte %00110110; ..XX.XX.
	.byte %00010000; ...X....

Car_Rotation_Frame3:
  ;.byte %00000000
  	.byte %00110000; ..XX....
  	.byte %00110010; ..XX.X..
	.byte %01100011; .XX...XX
  	.byte %01011110; .X.XXXX.
	.byte %11111111; XXXXXXXX
  	.byte %11000111; XX...XXX
  	.byte %00011100; ...XXX..
	.byte %00000100; .....X..


Car_Rotation_Frame4:;RIGHT|LEFT;OK
  ;.byte %00000000
	.byte %00000000; ........
	.byte %11101110; XXX.XXX.
	.byte %01100100; .XX..X..
	.byte %11011111; XX.XXXXX
	.byte %11011111; XX.XXXXX
	.byte %01100100; .XX..X..
	.byte %11101110; XXX.XXX.
	.byte %00000000; ........

Car_Rotation_Frame5:;OK
  ;.byte %00000000
	.byte %00000100; .....X..
	.byte %00011100; ...XXX..
	.byte %11000111; XX...XXX
	.byte %11111111; XXXXXXXX
	.byte %01011110; .X.XXXX.
	.byte %01100011; .XX...XX
	.byte %00110010; ..XX.X..
	.byte %00110000; ..XX....

Car_Rotation_Frame6:
  ;.byte %00000000
	.byte %00010000; ...X....
	.byte %00110110; ..XX.XX.
	.byte %00011110; ...XXXX.
	.byte %11011100; XX.XXX..
	.byte %11011011; XX.XX.XX
	.byte %01000010; .X....X.
	.byte %01111000; .XXXX...
	.byte %00011000; ...XX...

Car_Rotation_Frame7:
  ;.byte %00000000
	.byte %00101100; ..X.XX..
	.byte %00111100; ..XXXX..
	.byte %01011111; .X.XXXXX
	.byte %00011010; ...XX.X.
	.byte %11011010; XX.XX.X.
	.byte %11101000; XXX.X...
	.byte %00111100; ..XXXX..
	.byte %00001100; ....XX..



Car_Rotation_Frame8:;DOWN (ACTUAL)
  ;.byte %00000000
	.byte %00011000; ...XX...
	.byte %01011010; .X.XX.X.
	.byte %01111110; .XXXXXX.
	.byte %01011010; .X.XX.X.
	.byte %00011000; ...XX...
	.byte %01100110; .XX..XX.
	.byte %01111110; .XXXXXX.
	.byte %01011010; .X.XX.X.

Gremlin_Frame0_Left:;	OOOOWWAA
	.byte HMOVE_R1|BM_SIZE_2/4|0 ; .....XX. 2
	.byte HMOVE_R3|BM_SIZE_2/4|0 ; ....XX.. 1
	.byte HMOVE_L2|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_R2|BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_L3|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_R1|BM_SIZE_1/4|0 ; ....X... 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

Gremlin_Frame1_Left:;	OOOOWWAA
	.byte HMOVE_R1|BM_SIZE_2/4|0 ; ....XX.. 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_R2|BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_L3|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_R1|BM_SIZE_1/4|0 ; ....X... 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	
Gremlin_Frame2_Left:;	OOOOWWAA
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_R2|BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_L3|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_R1|BM_SIZE_1/4|0 ; ....X... 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

Gremlin_Frame3_Left:;	OOOOWWAA
	.byte HMOVE_L1|BM_SIZE_2/4|0 ; ..XX....-1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_R2|BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_L3|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_R1|BM_SIZE_1/4|0 ; ....X... 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

Gremlin_Frame4_Left:;	OOOOWWAA
	.byte HMOVE_L2|BM_SIZE_2/4|0 ; .XX.....-2
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_R2|BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_L3|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_R1|BM_SIZE_1/4|0 ; ....X... 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0






Gremlin_Frame0_Right:;	OOOOWWAA
	.byte HMOVE_L1|BM_SIZE_2/4|0 ; .XX..... 0
	.byte HMOVE_L1|BM_SIZE_2/4|0 ; ..XX.... 0
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_1/4|0 ; ...X.... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

Gremlin_Frame1_Right:;	OOOOWWAA
	.byte HMOVE_L1|BM_SIZE_2/4|0 ; ..XX.... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_1/4|0 ; ...X.... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

Gremlin_Frame2_Right:;	OOOOWWAA
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_1/4|0 ; ...X.... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0


Gremlin_Frame3_Right:;	OOOOWWAA
	.byte HMOVE_R1|BM_SIZE_2/4|0 ; ....XX.. 1
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_1/4|0 ; ...X.... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

Gremlin_Frame4_Right:;	OOOOWWAA
	.byte HMOVE_R4|BM_SIZE_2/4|0 ; .....XX.+2
	.byte HMOVE_L2|BM_SIZE_4/4|0 ; .XXXX...-2
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; . .XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_4/4|0 ; ...XXXX. 0
	.byte HMOVE_0 |BM_SIZE_1/4|0 ; ...X.... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0
	.byte HMOVE_0 |BM_SIZE_2/4|0 ; ...XX... 0

	Align 256

Score0_Digit_Table:
	.byte <Score_0-1;0
	.byte <Score_1-1;1
	.byte <Score_2-1;2
	.byte <Score_3-1
	.byte <Score_4-1
	.byte <Score_5-1
	.byte <Score_6-1
	.byte <Score_7-1
	.byte <Score_8-1
	.byte <Score_9-1
	
Score1_Digit_Table:
	.byte <Score_0-1 ;0
	.byte <Score_1b-1 ;1
	.byte <Score_2b-1 ;2
	.byte <Score_3b-1 
	.byte <Score_4b-1 
	.byte <Score_5b-1 
	.byte <Score_6b-1
	.byte <Score_7b-1 
	.byte <Score_8-1 
	.byte <Score_9b-1

Score_0:
	.byte %11101110
	.byte %10101010
	.byte %10101010
	.byte %10101010
	.byte %11101110

Score_1:
	.byte %01000100
	.byte %01000100
	.byte %01000100
	.byte %01000100
	.byte %01000100

Score_2:
	.byte %11101110
	.byte %10001000
	.byte %11101110
	.byte %00100010
	.byte %11101110



Score_3:
	.byte %11101110
	.byte %00100010
	.byte %01100110
	.byte %00100010
	.byte %11101110
	
Score_4:

	.byte %00100010
	.byte %00100010
	.byte %11101110
	.byte %10101010
	.byte %10101010
	
Score_5:
	.byte %11101110
	.byte %00100010
	.byte %11101110
	.byte %10001000
	.byte %11101110
	
Score_6:
	.byte %11101110
	.byte %10101010
	.byte %11101110
	.byte %10001000
	.byte %11101110


Score_7:

	.byte %00100010
	.byte %00100010
	.byte %00100010
	.byte %00100010
	.byte %11101110


Score_8:
	.byte %11101110
	.byte %10101010
	.byte %11101110
	.byte %10101010
	.byte %11101110


Score_9:
	.byte %00100010
	.byte %00100010
	.byte %11101110
	.byte %10101010
	.byte %11101110

	

Score_1b:

	.byte %01000100
	.byte %01000100
	.byte %01000100
	.byte %01000100
	.byte %01000100

	
Score_2b:
	.byte %11101110
	.byte %00100010
	.byte %11101110
	.byte %10001000
	.byte %11101110

Score_3b:
	.byte %11101110
	.byte %10001000
	.byte %11001100
	.byte %10001000
	.byte %11101110

	
Score_4b:

	.byte %10001000
	.byte %10001000
	.byte %11101110
	.byte %10101010
	.byte %10101010


Score_5b:
	.byte %11101110
	.byte %10001000
	.byte %11101110
	.byte %00100010
	.byte %11101110



Score_6b:
	.byte %11101110
	.byte %10101010
	.byte %11101110
	.byte %00100010
	.byte %11101110
	
Score_7b:

	.byte %10001000
	.byte %10001000
	.byte %10001000
	.byte %10001000
	.byte %11101110


Score_9b:
	.byte %10001000
	.byte %10001000
	.byte %11101110
	.byte %10101010
	.byte %11101110




;						X	Y
X_Fraction_Table:	;				 
	.byte 0		;0  - point up		0	-	use these 4 only for X
	.byte 48	;1  - 			+	-
	.byte 80	;2  - 			+	-
	.byte 144	;3  - 			+	-
	.byte 192	;4  - point right	+	0
	.byte 144	;5  - 			+	+
	.byte 80	;6  - 			+	+
	.byte 48	;7  - 			+	+
	;X CUTOFF
	.byte 0		;8  - point down	0	+
	.byte 48	;9  - 			-	+
	.byte 80	;10  - 			-	+
	.byte 144	;11  - 			-	+
	.byte 192	;12  - point left	-	0

	.byte 144	;13  - 			-	-
	.byte 80	;14  - 			-	-
	.byte 48	;15  - 			-	-
	


Y_Fraction_Table:
	.byte 255	;4  - point right	+	0
	.byte 191	;5  - 			+	+
	.byte 127	;6  - 			+	+
	.byte 63	;7  - 			+	+
	;X CUTOFF
	.byte 0		;8  - point down	0	+
	.byte 63	;9  - 			-	+
	.byte 127	;10  - 			-	+
	.byte 191	;11  - 			-	+
	.byte 255	;12  - point left	-	0

	.byte 191	;13  - 			-	-
	.byte 127	;14  - 			-	-
	.byte 63	;15  - 			-	-
	
	.byte 0		;0  - point up		0	-
	.byte 63	;1  - 			+	-
	.byte 127	;2  - 			+	-
	.byte 191	;3  - 			+	-



Border_Type_Table:
	.byte $32; red 	- solid
	.byte $1D; yellow 	- crashthru
	.byte $CD; green 	- passthru
;for greyscale
	.byte $0F; white 	- solid
	.byte $06; grey 	- crashthru
	.byte $02; dark grey	- passthru
	
Grenade_Size_Table:
	.byte BM_SIZE_8 ; powerups are big
	.byte BM_SIZE_2 ; grenades are small
	.byte BM_SIZE_1 ; easter egg
	.byte BM_SIZE_2 ; barrier
	
Grenade_Color_Table:
	.byte $56; purple 	- powerup
	.byte $D4; olive 	- grenade
	.byte $00; black 	- easter egg
	.byte $0F; ; barrier	
	
;for greyscale
	.byte $0F; white 	- powerup
	.byte $06; grey	 - grenade
	.byte $04; grey 	- easter egg (give those with B&W TVs an advantage)
	.byte $0F; ; barrier	

Score_Gradient:
	.byte $05
	.byte $07
	.byte $09
	.byte $0B
	.byte $0F


Scream_Volume_Sweep:
	.byte	15,15,15,15,15,14,13,12,11,10,11,10,9,8,7,6,5,4,3,3,2,2,1,1,0,0

	
INPT:
	.byte INPT5
	.byte INPT4



Engine_Noise_Type:
	.byte	2;idle
	.byte	15;full speed
	
Engine_Noise_Pitch:
	.byte	9;idle
	.byte	15;full speed
	
Engine_Noise_Volume:
	.byte	6;idle
	.byte	15;full speed


Next_DC_Table:
	.byte %01, %11, %00, %10
	
	org $FFFC;4K ROM
	.word	Start
	.word	Start
